<?php
/**
* @author Amasty Team
* @copyright Copyright (c) 2022 Amasty (https://www.amasty.com)
* @package Free Gift for Magento 2
*/
declare(strict_types=1);

namespace Amasty\Promo\Test\Unit\Model\ItemRegistry;

use Amasty\Promo\Model\ItemRegistry\PromoItemData;
use Amasty\Promo\Model\ItemRegistry\PromoItemRemover;
use Amasty\Promo\Model\ResourceModel\Rule;
use PHPUnit\Framework\MockObject\MockObject;
use PHPUnit\Framework\TestCase;

/**
 * @see PromoItemRemover
 */
class PromoItemRemoverTest extends TestCase
{
    private const RULE_ID = 1;

    /**
     * @var Rule|MockObject
     */
    private $ruleMock;

    /**
     * @var PromoItemRemover
     */
    private $subject;

    public function setUp(): void
    {
        $this->ruleMock = $this->createMock(Rule::class);
        $this->subject = new PromoItemRemover($this->ruleMock);
    }

    /**
     * @covers PromoItemRemover::execute
     *
     * @param string[] $initialItemsSkus
     * @param array[] $ruleSkus
     * @param string[] $resultItemsSkus
     * @return void
     *
     * @dataProvider executeDataProvider
     */
    public function testExecute(array $initialItemsSkus, array $ruleSkus, array $resultItemsSkus): void
    {
        $initialItems = [];
        $resultItems = [];
        foreach ($initialItemsSkus as $sku) {
            $initialItems[] = $this->initItem($sku);
        }
        foreach ($resultItemsSkus as $sku) {
            $resultItems[] = $this->initItem($sku);
        }

        $this->ruleMock->expects($this->once())->method('isApplicable')->willReturn($ruleSkus);

        $this->assertEquals($resultItems, $this->subject->execute($initialItems));
    }

    /**
     * @return array[]
     */
    public function executeDataProvider(): array
    {
        return [
            [[], [], []],
            [['sku1', 'sku2', 'sku3'], [0 => ['sku' => 'sku1,sku2,sku3']], ['sku1', 'sku2', 'sku3']],
            [['sku1', 'sku2', 'sku3'], [0 => ['sku' => 'sku1,sku2']], ['sku1', 'sku2']]
        ];
    }

    private function initItem(string $sku): PromoItemData
    {
        return $this->createConfiguredMock(PromoItemData::class, ['getSku' => $sku, 'getRuleId' => self::RULE_ID]);
    }
}
